import { CodeEditorView, SourceCode } from './code-editor.view';
import { BehaviorSubject, ReplaySubject } from 'rxjs';
import * as ts from 'typescript';
import { VirtualDOM } from '@youwol/flux-view';
declare type SourcePath = string;
export declare class CodeIdeState {
    readonly entryPoint: SourcePath;
    readonly currentFile$: BehaviorSubject<SourceCode>;
    readonly fsMap$: BehaviorSubject<Map<string, string>>;
    readonly parsedSrc$: ReplaySubject<{
        jsSrc: string;
        tsSrc: string;
    }>;
    readonly config: {
        lineNumbers: boolean;
        theme: string;
        lineWrapping: boolean;
        gutters: string[];
        indentUnit: number;
        lint: {
            options: {
                editorKind: string;
                esversion: number;
            };
        };
        extraKeys: {
            'Ctrl-Enter': () => void;
        };
    };
    constructor(params: {
        files: SourceCode[] | SourceCode;
        entryPoint: SourcePath;
    });
    parseCurrentFile$(): import("rxjs").Observable<{
        tsSrc: string;
        jsSrc: string;
    }>;
}
export declare class CodeIdeView implements VirtualDOM {
    readonly class = "d-flex h-100";
    readonly children: VirtualDOM[];
    readonly ideState: CodeIdeState;
    readonly tsCodeEditorView: TsCodeEditorView;
    constructor(params: {
        ideState: CodeIdeState;
    });
}
export declare class TsCodeEditorView extends CodeEditorView {
    readonly ideState: CodeIdeState;
    constructor(params: {
        ideState: CodeIdeState;
    });
}
export interface SrcPosition {
    line: number;
    ch: number;
}
export declare class SrcHighlight {
    readonly diagnostic: ts.Diagnostic;
    readonly messageText: string;
    readonly from: SrcPosition;
    readonly to: SrcPosition;
    constructor(diagnostic: ts.Diagnostic);
}
export declare const compilerOptions: {
    target: ts.ScriptTarget;
    module: ts.ModuleKind;
    esModuleInterop: boolean;
    noImplicitAny: boolean;
    baseUrl: string;
};
export declare function getHighlights(fsMap: any, src: any): SrcHighlight[];
export {};
